#!/bin/bash
#
# Install bodhi's uBoot and debian rootfs on Kirkwood devices

# Copyright (c) 2016 Alberto Bursi
# Copyright (c) 2012 Jeff Doozan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# Version 2.0    [2016] Revised/extended by Alberto Bursi to cover more devices
#                       and to be run with its own toolbox
# Version 1.1    [1/24/2013] Revised by Daniel Richard G.
# Version 1.0    [7/14/2012] Initial Release

##############################
####
#### Definitions used throughout the script
####
##############################

# Download locations
MIRROR="https://dl.dropboxusercontent.com/u/47541136/linux/tools_for_Kirkwood_installer"

#http://download.doozan.com/debian/mke2fs"

# Where we download and keep tools temporarily
TOOLS="/tmp/tools"

# Tools and their download link
#BLPARAM_URL="$MIRROR/blparam"
NANDDUMP_URL="$MIRROR/nanddump"
NANDWRITE_URL="$MIRROR/nandwrite"
FLASH_ERASE_URL="$MIRROR/flash_erase"
FW_PRINTENV_URL="$MIRROR/fw_printenv"
FW_CONFIG_URL="$MIRROR/fw_env.config"

BUSYBOX_URL="$MIRROR/busybox"
MKE2FS_URL="$MIRROR/mke2fs"
E2FSCK_URL="$MIRROR/e2fsck"
FDISK_URL="$MIRROR/fdisk"

WGET_WITH_SSL_URL="http://pogoplug.s3.amazonaws.com/wget"

# MD5 urls
GENERIC_MD5_URL=$MIRROR
#BLPARAM_MD5="$GENERIC_MD5_URL/blparam.md5"
NANDDUMP_MD5="$GENERIC_MD5_URL/nanddump.md5"
NANDWRITE_MD5="$GENERIC_MD5_URL/nandwrite.md5"
FLASH_ERASE_MD5="$GENERIC_MD5_URL/flash_erase.md5"
FW_PRINTENV_MD5="$GENERIC_MD5_URL/fw_printenv.md5"
#FW_SETENV_MD5="$GENERIC_MD5_URL/fw_setenv.md5" this uses the same binary as fw_printenv
FW_CONFIG_MD5="$GENERIC_MD5_URL/fw_env.config.md5"

BUSYBOX_MD5="$GENERIC_MD5_URL/busybox.md5"
MKE2FS_MD5="$GENERIC_MD5_URL/mke2fs.md5"
E2FSCK_MD5="$GENERIC_MD5_URL/e2fsck.md5"
FDISK_MD5="$GENERIC_MD5_URL/fdisk.md5"

#WGET_WITH_SSL_MD5="$GENERIC_MD5_URL/wget.md5" not used as we cannot download md5 for it for now

#uboot md5 used to check if current uBoot is recognized/valid or something
#VALID_UBOOT_MD5="http://jeff.doozan.com/uboot/valid-uboot.md5"

# Tools and their place on disk
#BLPARAM="$TOOLS/blparam"
NANDDUMP="$TOOLS/nanddump"
NANDWRITE="$TOOLS/nandwrite"
FLASH_ERASE="$TOOLS/flash_erase"
FW_PRINTENV="$TOOLS/fw_printenv"
FW_SETENV="$TOOLS/fw_setenv"
FW_CONFIG="$TOOLS/etc/fw_env.config"

BUSYBOX="$TOOLS/busybox"
MKE2FS="$TOOLS/mke2fs"
E2FSCK="$TOOLS/e2fsck"
FDISK="$TOOLS/fdisk"

WGET_WITH_SSL="$TOOLS/wget"


#rootfs link and md5 link (Debian-4.4.0-kirkwood-tld-1-rootfs-bodhi.tar.bz2)
ROOTFS_URL="https://bitly.com/20KHWq5"
ROOTFS_MD5="$GENERIC_MD5_URL/Debian-4.4.0-kirkwood-tld-1-rootfs-bodhi.tar.bz2.md5"

ROOTFS_TAR="Debian-4.4.0-kirkwood-tld-1-rootfs-bodhi.tar"
ROOTFS="/tmp/rootfs/$ROOTFS_TAR.bz2"

#Device-specific definitions and commands are at the end of all this, in the MAIN, inside the "case" statement



##############################
####
#### A bunch of functions used by the script since we need to reuse the same stuff over and over for dozens
#### of different Kirkwood devices
####
##############################

verify_md5 ()
{
  file="$1"
  md5="$2"

  echo "verify file $file with md5 $md5"

  check_md5=$(cat "$md5" | cut -d' ' -f1)

  echo $check_md5

  file_md5=$(md5sum "$file" | cut -d' ' -f1)

  echo $file_md5

  if [ "$check_md5" = "$file_md5" ]; then
    echo "passed"
    return 0
  else
    echo "failed"
    return 1

  fi
}

download_and_verify () # add curl as a fallback     curl -O http://server1.cyberciti.biz/file.tar.gz
{
  file_dest="$1"
  file_url="$2"

  #now this comes from outside as we keep md5 on a different server
  md5_url="$3"

  md5_dest="$file_dest.md5"

  echo $file_dest
  echo $file_url
  echo $md5_url
  echo $md5_dest


  # Always download a fresh MD5, in case a newer version is available
  # the no-check-certificate in wget is needed because 99% of the cases the stock firmware will lack
  # a "legit certificates" list, so we must ask wget to not check the ssl certificate's validity.
  if [ -f "$md5_dest" ]; then rm -f "$md5_dest"; fi
  wget -O "$md5_dest" "$md5_url" --no-check-certificate
  # retry the download if it failed
  if [ ! -f "$md5_dest" ]; then
    wget -O  "$md5_dest" "$md5_url" --no-check-certificate
    if [ ! -f "$md5_dest" ]; then
      return 1 # Could not get md5
    fi
  fi

  # If the file already exists, check the MD5
 # if [ -f "$file_dest" ]; then
 #   verify_md5 "$file_dest" "$md5_dest"
 #   if [ "$?" -ne "0" ]; then
 #     rm -f "$md5_dest"
 #     return 0
 #   else
 #     rm -f "$file_dest"
 #   fi
 #fi

  # Download the file
  wget -O  "$file_dest" "$file_url" --no-check-certificate
  # retry the download if it failed
  verify_md5 "$file_dest" "$md5_dest"
  if [ "$?" -ne "0" ]; then
    echo "Download failed or MD5 did not match, try again"
    if [ -f "$file_dest" ]; then rm -f "$file_dest"; fi
    wget -O  "$file_dest" "$file_url" --no-check-certificate
    verify_md5 "$file_dest" "$md5_dest"
    if [ "$?" -ne "0" ]; then
      rm -f "$md5_dest"
      return 1
    fi
  fi

  rm -f "$md5_dest"
  return 0
}

install ()
{
  file_dest="$1"
  file_url="$2"
  file_md5_url="$3" # md5 file url
  file_pmask="$4"  # Permissions mask

  #echo "# checking for $file_dest..."

  # Install target file if it doesn't already exist
  #if [ ! -s "$file_dest" ]; then
    echo ""
    echo "# Installing $file_dest..."

    #####DISABLED as we dump all tools in a tmpfs anyway
    # Check for read-only filesystem by testing
    #  if we can delete the existing 0 byte file
    #  or, if we can create a 0 byte file
    #is_readonly=0
    #if [ -f "$file_dest" ]; then
    #  rm -f "$file_dest" 2> /dev/null
    #else
    #  touch "$file_dest" 2> /dev/null
    #fi
    #if [ "$?" -ne "0" ]; then
    #  is_readonly=0
    #  mount -o remount,rw /
    #fi
    #rm -f "$file_dest" 2> /dev/null

    if [[ "$file_md5_url" == "NO_CHECK" ]]; then
      # Download the file without checking md5
      wget -O  "$file_dest" "$file_url" --no-check-certificate
    fi

    if [[ "$file_md5_url" != "NO_CHECK" ]]; then
      download_and_verify "$file_dest" "$file_url" "$file_md5_url"
      if [ "$?" -ne "0" ]; then
        echo "## Could not install $file_dest from $file_url, exiting."
    #    if [ "$is_readonly" = "1" ]; then
    #      mount -o remount,ro /
    #    fi
        exit 1
      fi
    fi

   echo " chmodding ""$file_pmask" " -- " "$file_dest"
    chmod "$file_pmask" "$file_dest"

   # if [ "$is_readonly" = "1" ]; then
   #   mount -o remount,ro /
   # fi

    echo "# Successfully installed $file_dest."
 # fi

  return 0
}


download_tools() {

#for the sake of making sure we can execute them, we remount /tmp as exec
mount -o remount,exec "/tmp"

# Create the tools and TOOLS/etc directory if it doesn't already exist
mkdir -p "$TOOLS/etc"

#downloading manually a wget that can download from https links
wget -O  "$WGET_WITH_SSL"           "$WGET_WITH_SSL_URL"
chmod "755" "$WGET_WITH_SSL"

#downloading busybox
install "$BUSYBOX"          "$BUSYBOX_URL"      "NO_CHECK"         755

#backing up the standard PATH so we can use it later too.
ORIGINAL_PATH="$PATH"

#changing PATH so we call stuff from our own folders from now on
#placing it before the stock path so all commands will be first called from here
#first goes the tools directory so we should call the right wget
export PATH="$TOOLS"

echo "new PATH is now $PATH"

#adding symlinks to all the tools
#could have done a cooler way but I'm keeping this simple and brutal as stock firmware's shell may not like it
cd $TOOLS #moving into the folder.
$BUSYBOX ln -s $BUSYBOX ln #first is busybox's own symlink making a symlink for itself, then we go on

ln -s $BUSYBOX [
ln -s $BUSYBOX [[
ln -s $BUSYBOX acpid
ln -s $BUSYBOX addgroup
ln -s $BUSYBOX adduser
ln -s $BUSYBOX adjtimex
ln -s $BUSYBOX ar
ln -s $BUSYBOX arping
ln -s $BUSYBOX ash
ln -s $BUSYBOX awk
ln -s $BUSYBOX basename
ln -s $BUSYBOX blockdev
ln -s $BUSYBOX brctl
ln -s $BUSYBOX bunzip2
ln -s $BUSYBOX bzcat
ln -s $BUSYBOX bzip2
ln -s $BUSYBOX cal
ln -s $BUSYBOX cat
ln -s $BUSYBOX chgrp
ln -s $BUSYBOX chmod
ln -s $BUSYBOX chown
ln -s $BUSYBOX chroot
ln -s $BUSYBOX chvt
ln -s $BUSYBOX clear
ln -s $BUSYBOX cmp
ln -s $BUSYBOX cp
ln -s $BUSYBOX cpio
ln -s $BUSYBOX crond
ln -s $BUSYBOX crontab
ln -s $BUSYBOX cut
ln -s $BUSYBOX date
ln -s $BUSYBOX dc
ln -s $BUSYBOX dd
ln -s $BUSYBOX deallocvt
ln -s $BUSYBOX delgroup
ln -s $BUSYBOX deluser
ln -s $BUSYBOX df
ln -s $BUSYBOX dirname
ln -s $BUSYBOX dmesg
ln -s $BUSYBOX dnsdomainname
ln -s $BUSYBOX dos2unix
ln -s $BUSYBOX dpkg
ln -s $BUSYBOX  dpkg-deb
ln -s $BUSYBOX  du
ln -s $BUSYBOX  dumpkmap
ln -s $BUSYBOX  dumpleases
ln -s $BUSYBOX  echo
ln -s $BUSYBOX  ed
ln -s $BUSYBOX  egrep
ln -s $BUSYBOX  eject
ln -s $BUSYBOX  env
ln -s $BUSYBOX  expand
ln -s $BUSYBOX  expr
ln -s $BUSYBOX  false
ln -s $BUSYBOX  fbset
ln -s $BUSYBOX  fdflush
ln -s $BUSYBOX  fdisk
ln -s $BUSYBOX  fgrep
ln -s $BUSYBOX  find
ln -s $BUSYBOX  fold
ln -s $BUSYBOX  free
ln -s $BUSYBOX  freeramdisk
ln -s $BUSYBOX  fsck.minix
ln -s $BUSYBOX  ftpget
ln -s $BUSYBOX  ftpput
ln -s $BUSYBOX  getopt
ln -s $BUSYBOX  getty
ln -s $BUSYBOX  grep
ln -s $BUSYBOX  gunzip
ln -s $BUSYBOX  gzip
ln -s $BUSYBOX  halt
ln -s $BUSYBOX  head
ln -s $BUSYBOX  hexdump
ln -s $BUSYBOX  hostid
ln -s $BUSYBOX  hostname
ln -s $BUSYBOX  httpd
ln -s $BUSYBOX  hwclock
ln -s $BUSYBOX  id
ln -s $BUSYBOX  ifconfig
ln -s $BUSYBOX  ifdown
ln -s $BUSYBOX  ifup
ln -s $BUSYBOX  init
ln -s $BUSYBOX  ionice
ln -s $BUSYBOX  ip
ln -s $BUSYBOX  ipcalc
ln -s $BUSYBOX  kill
ln -s $BUSYBOX  killall
ln -s $BUSYBOX  klogd
ln -s $BUSYBOX  last
ln -s $BUSYBOX  length
ln -s $BUSYBOX  less
ln -s $BUSYBOX  linuxrc
ln -s $BUSYBOX  loadfont
ln -s $BUSYBOX  loadkmap
ln -s $BUSYBOX  logger
ln -s $BUSYBOX  login
ln -s $BUSYBOX  logname
ln -s $BUSYBOX  logread
ln -s $BUSYBOX  losetup
ln -s $BUSYBOX  ls
ln -s $BUSYBOX  lzcat
ln -s $BUSYBOX  makedevs
ln -s $BUSYBOX  md5sum
ln -s $BUSYBOX  mdev
ln -s $BUSYBOX  mesg
ln -s $BUSYBOX  microcom
ln -s $BUSYBOX  mkdir
ln -s $BUSYBOX  mkfifo
ln -s $BUSYBOX  mkfs.minix
ln -s $BUSYBOX  mknod
ln -s $BUSYBOX  mkswap
ln -s $BUSYBOX  mktemp
ln -s $BUSYBOX  more
ln -s $BUSYBOX  mount
ln -s $BUSYBOX  mt
ln -s $BUSYBOX  mv
ln -s $BUSYBOX  nameif
ln -s $BUSYBOX  nc
ln -s $BUSYBOX  netstat
ln -s $BUSYBOX  nslookup
ln -s $BUSYBOX  od
ln -s $BUSYBOX  openvt
ln -s $BUSYBOX  passwd
ln -s $BUSYBOX  patch
ln -s $BUSYBOX  pidof
ln -s $BUSYBOX  ping
ln -s $BUSYBOX  ping6
ln -s $BUSYBOX  pivot_root
ln -s $BUSYBOX  poweroff
ln -s $BUSYBOX  printf
ln -s $BUSYBOX  ps
ln -s $BUSYBOX  pwd
ln -s $BUSYBOX  rdate
ln -s $BUSYBOX  readlink
ln -s $BUSYBOX  realpath
ln -s $BUSYBOX  reboot
ln -s $BUSYBOX  renice
ln -s $BUSYBOX  reset
ln -s $BUSYBOX  rm
ln -s $BUSYBOX  rmdir
ln -s $BUSYBOX  route
ln -s $BUSYBOX  rpm
ln -s $BUSYBOX  rpm2cpio
ln -s $BUSYBOX  run-parts
ln -s $BUSYBOX  sed
ln -s $BUSYBOX  setkeycodes
ln -s $BUSYBOX  sh
ln -s $BUSYBOX  sha1sum
ln -s $BUSYBOX  sha256sum
ln -s $BUSYBOX  sha512sum
ln -s $BUSYBOX  sleep
ln -s $BUSYBOX  sort
ln -s $BUSYBOX  start-stop-daemon
ln -s $BUSYBOX  strings
ln -s $BUSYBOX  stty
ln -s $BUSYBOX  su
ln -s $BUSYBOX  sulogin
ln -s $BUSYBOX  swapoff
ln -s $BUSYBOX  swapon
ln -s $BUSYBOX  sync
ln -s $BUSYBOX  syslogd
ln -s $BUSYBOX  tac
ln -s $BUSYBOX  tail
ln -s $BUSYBOX  tar
ln -s $BUSYBOX  tee
ln -s $BUSYBOX  telnet
ln -s $BUSYBOX  telnetd
ln -s $BUSYBOX  test
ln -s $BUSYBOX  tftp
ln -s $BUSYBOX  time
ln -s $BUSYBOX  timeout
ln -s $BUSYBOX  top
ln -s $BUSYBOX  touch
ln -s $BUSYBOX  tr
ln -s $BUSYBOX  traceroute
ln -s $BUSYBOX  traceroute6
ln -s $BUSYBOX  true
ln -s $BUSYBOX  tty
ln -s $BUSYBOX  tunctl
ln -s $BUSYBOX  udhcpc
ln -s $BUSYBOX  udhcpd
ln -s $BUSYBOX  umount
ln -s $BUSYBOX  uname
ln -s $BUSYBOX  uncompress
ln -s $BUSYBOX  unexpand
ln -s $BUSYBOX  uniq
ln -s $BUSYBOX  unix2dos
ln -s $BUSYBOX  unlzma
ln -s $BUSYBOX  unzip
ln -s $BUSYBOX  uptime
ln -s $BUSYBOX  usleep
ln -s $BUSYBOX  uudecode
ln -s $BUSYBOX  uuencode
ln -s $BUSYBOX  vconfig
ln -s $BUSYBOX  vi
ln -s $BUSYBOX  vlock
ln -s $BUSYBOX  watch
ln -s $BUSYBOX  watchdog
ln -s $BUSYBOX  wc
#ln -s $BUSYBOX  wget
ln -s $BUSYBOX  which
ln -s $BUSYBOX  who
ln -s $BUSYBOX  whoami
ln -s $BUSYBOX  xargs
ln -s $BUSYBOX  yes
ln -s $BUSYBOX  zcat
#whew, done.

chmod -R "755" $TOOLS

ls -l

cd

#install "$FDISK"           "$FDISK_URL"       "$FDISK_MD5"          755 it cannot deal with GPT disks so it is useless
install "$E2FSCK"           "$E2FSCK_URL"       "$E2FSCK_MD5"          755
install "$MKE2FS"           "$MKE2FS_URL"       "$MKE2FS_MD5"          755
install "$NANDWRITE"        "$NANDWRITE_URL"    "$NANDWRITE_MD5"       755
install "$NANDDUMP"         "$NANDDUMP_URL"     "$NANDDUMP_MD5"        755
install "$FLASH_ERASE"      "$FLASH_ERASE_URL"  "$FLASH_ERASE_MD5"     755
install "$FW_PRINTENV"      "$FW_PRINTENV_URL"  "$FW_PRINTENV_MD5"     755

if [ ! -f "$FW_SETENV" ]; then
  ln -s "$FW_PRINTENV" "$FW_SETENV" 2> /dev/null
 # if [ "$?" -ne "0" ]; then
 #   mount -o remount,rw /
 #   ln -s "$FW_PRINTENV" "$FW_SETENV"
 #   mount -o remount,ro /
 # fi
fi

install "$FW_CONFIG"        "$FW_CONFIG_URL"    "$FW_CONFIG_MD5"       644

}

run_fw_printenv_setenv_in_chroot(){
command=$1

#preparing folder mounts for chroot

mkdir -p "$TOOLS/proc"
mount proc "$TOOLS/proc" -t proc

mkdir -p "$TOOLS/sysfs"
mount proc "$TOOLS/sysfs" -t sysfs

mkdir -p "$TOOLS/dev"
mount -o bind "/dev" "$TOOLS/dev"

#chrooting to give the command only

chroot $TOOLS/ ./$command

#unmounting stuff
umount "$TOOLS/proc"
umount "$TOOLS/sysfs"
umount "$TOOLS/dev"

}

#need to make a fallback, so that if flashing uboot OR the envs fails we can reflash the stock uboot
install_uboot() {

# Install uBoot on /dev/mtd0

# the main part of the download link
MIRROR=http://download.doozan.com/uboot

#u-boot-bodhi-latest.<platform>.mtd0.kwb
#u-boot-bodhi-latest.<platform>.mtd0.kwb.md5
#u-boot-bodhi-latest.<platform>.mtd0.kwb.sha256
#debian-kirkwood-rootfs-bodhi-latest.tar.bz2
#debian-kirkwood-rootfs-bodhi-latest.tar.bz2.md5
#debian-kirkwood-rootfs-bodhi-latest.tar.bz2.sha256

UBOOT_MTD0_BASE_URL="$MIRROR/u-boot-bodhi-latest" # .platform.mtd0.kwb will be appended to this
UBOOT_ENV_URL="$MIRROR/files/environment/uboot.environment"

UBOOT_MTD0="/tmp/uboot.mtd0.kwb"
UBOOT_ORIGINAL="/uboot-original-mtd0.kwb"
UBOOT_ENV="/tmp/uboot.environment"

#hardcoding this to trigger the if below
UPDATE_UBOOT=1

# Download and install the latest uBoot
if [ "$UPDATE_UBOOT" = "1" ]; then

  echo ""
  echo "# Installing uBoot"

  #u-boot-bodhi-latest.<platform>.mtd0.kwb
  UBOOT_MTD0_URL="$UBOOT_MTD0_BASE_URL.$UBOOT_PLATFORM.mtd0.kwb"

  echo "## Installing $UBOOT_PLATFORM u-boot-bodhi-latest.$UBOOT_PLATFORM.mtd0.kwb"

  if [ -f "$UBOOT_MTD0" ]; then rm "$UBOOT_MTD0"; fi

  download_and_verify "$UBOOT_MTD0" "$UBOOT_MTD0_URL"

  if [ "$?" -ne "0" ]; then
    echo "## uBoot could not be downloaded, or the MD5 does not match."
    echo "## Exiting. No changes were made to mtd0."
    exit 1
  fi

  # hardcoding this for now, having this as a variable is more flexible
  # like for example in case we are installing from Debian
  DEV_MTD0="$STOCK_MTD0"

  # Write new uBoot to mtd0
  # Erase the first 512k
  "$FLASH_ERASE" "$DEV_MTD0" 0 4
  #need to add a check on the output here.

  "$NANDWRITE" "$DEV_MTD0" "$UBOOT_MTD0"
  #need to add a check on the output here.

  # dump mtd0 and compare the checksum, to make sure it installed properly
  "$NANDDUMP" --noecc --omitoob -l 0x80000 -f /tmp/mtd0.uboot "$DEV_MTD0"
  echo "## Verifying new uBoot..."
  if [ -f "$UBOOT_MTD0.md5" ]; then rm "$UBOOT_MTD0.md5"; fi
  wget -O "$UBOOT_MTD0.md5" "$UBOOT_MTD0_URL.md5"

  verify_md5 "/tmp/mtd0.uboot" "$UBOOT_MTD0.md5"
  if [ "$?" -ne "0" ]; then
    rm -f "$UBOOT.md5"
    echo "##"
    echo "##"
    echo "## VERIFICATION FAILED!"
    echo "##"
    echo "## uBoot was not properly installed to mtd0."
    echo "##"
    echo "##"
    echo "## YOUR DEVICE MAY BE IN AN UNUSABLE STATE."
    echo "## DO NOT REBOOT OR POWER OFF YOUR DEVICE"
    echo "##"
    echo "##"
    echo "## Make a backup of /tmp/uboot-mtd0-dump someplace safe and"
    echo "## then re-run this installer."
    exit 1
  else
    echo "# Verified successfully!"
  fi
  rm -f "$UBOOT_MTD0.md5"

fi

}

install_envs() {
#now receives the results of the checks to see if we need to update envs too

#bunch of code that we don't need until we enable uboot md5 checking

#if [ "$UPDATE_UBOOT" = "0" ]; then
#  UPDATE_UBOOT_ENVIRONMENT=0
#elif [ "$UBOOT_VERSION" = "original" -o "$UBOOT_VERSION" = "unknown" ]; then
#  UPDATE_UBOOT_ENVIRONMENT=1
#else  # Updating uboot, but it's from a known "new" uboot, no need to reset the environment
#  UPDATE_UBOOT_ENVIRONMENT=0
#fi

#if [ "$UPDATE_UBOOT_ENVIRONMENT" = "0" -a "$1" ]; then
#  echo ""
#  echo ""
#  if [ "$UPDATE_UBOOT" = "0" ]; then
#    echo "You are already running the latest uBoot."
#  else
#    echo "You are already running a modern uBoot."
#  fi
#  echo "Your current uBoot environment should be reasonable.  However, if you're having"
#  echo "any probems booting, you can reset the environment variables to known good values."
#  echo -n "Would you like to reset the uBoot environment? [N/y] "
#  read PROMPT
#  if [ "$PROMPT" = "y" -o "$PROMPT" = "Y" ]; then
#    UPDATE_UBOOT_ENVIRONMENT=1
#  fi
#fi

echo "We are now ready to flash uboot environment, its boot configuration."
echo ""
echo "If this is the first time you run this script, answer Y"
echo ""
echo "If you are updating a previous version of bodhi's uboot and have"
echo "custom uboot envs you wish to keep, answer N"
echo ""
echo ""
echo "Would you like to install the new uboot environment? [N/y] "
read -r PROMPT
  if [ "$PROMPT" = "y" ] || [ "$PROMPT" = "Y" ]; then
    UPDATE_UBOOT_ENVIRONMENT=1
  fi

if [ "$UPDATE_UBOOT_ENVIRONMENT" = "1" ]; then
  echo ""
  echo "# Installing uBoot environment"


  # Install the uBoot environment
  download_and_verify "$UBOOT_ENV" "$UBOOT_ENV_URL"
  if [ "$?" -ne "0" ]; then
    echo "## Could not install uBoot environment, exiting"
    exit 1
  fi
  "$FLASH_ERASE" "$DEV_MTD0" 0xc0000 1
  "$NANDWRITE" "-s" "786432" "$DEV_MTD0" "$UBOOT_ENV"

  echo ""
  echo "# Verifying uBoot environment"

  # Verify the uBoot environment
  "$NANDDUMP" --noecc --omitoob -f "/tmp/uboot.environment" -s 0xc0000 -l 0x20000 "$DEV_MTD0"
  wget -O "$UBOOT_ENV.md5" "$UBOOT_ENV_URL.md5"
  verify_md5 "/tmp/uboot.environment" "$UBOOT_ENV.md5"
  if [ "$?" -ne "0" ]; then
    rm -f "$UBOOT_ENV.md5"
    echo "## VERIFICATION FAILED!"
    echo "## uBoot environment was not properly written to mtd0.  Please re-run this installer."
    exit 1
  fi
  rm -f "$UBOOT_ENV.md5"

fi


echo ""
echo "# uBoot installation has completed successfully."

} #end of update_envs


preserve_crucial_envs_for_later(){

#preserving crucial envs so we can add them again later

  # Preserve the MAC address
  ENV_ETHADDR=`$FW_PRINTENV ethaddr 2> /dev/null | cut -d'=' -f 2-`

  # Preserve the mtdparts
  ENV_MTDPARTS=`$FW_PRINTENV mtdparts 2> /dev/null | cut -d'=' -f 2-`

}

preserve_custom_envs_for_later(){

#this is still WIP, currently disabled

  # Preserve the 'rescue_installed' setting
  ENV_RESCUE_INSTALLED=`$FW_PRINTENV rescue_installed 2> /dev/null | cut -d'=' -f 2-`
  if [ "$ENV_RESCUE_INSTALLED" = "" ]; then
    ENV_BOOTCMD_RESCUE=`$FW_PRINTENV bootcmd_rescue 2> /dev/null`
    if [ "$ENV_BOOTCMD_RESCUE" != "" ]; then
      ENV_RESCUE_INSTALLED=1
    fi
  fi

  # Preserve the arcNumber value
  ENV_ARCNUMBER=`$FW_PRINTENV arcNumber 2> /dev/null | cut -d'=' -f 2-`

  # Preserve the custom kernel parameters
  ENV_RESCUE_CUSTOM=`$FW_PRINTENV rescue_custom_params 2> /dev/null | cut -d'=' -f 2-`
  ENV_USB_CUSTOM=`$FW_PRINTENV usb_custom_params 2> /dev/null | cut -d'=' -f 2-`
  ENV_UBIFS_CUSTOM=`$FW_PRINTENV ubifs_custom_params 2> /dev/null | cut -d'=' -f 2-`
}

set_crucial_envs(){

#writing all envs that MUST be set for the box to boot/work at all

# writing MAC back
"$FW_SETENV" "ethaddr" "$ENV_ETHADDR"
# writing mtdparts back
"$FW_SETENV" "mtdparts" "$ENV_MTDPARTS"

# writing dtb
"$FW_SETENV" "dtb_file" "$ENV_DTB_FILE"

# writing a workaround to make sure boot from Sata works fine
# This will help the "ide reset" to work properly.
# There seems to be a bug in u-boot that if you do "ide reset" too quickly in succession,
# the SATA drive might have problem spinning up
$FW_SETENV "bootcmd_uenv" "'run uenv_load; if test $uenv_loaded -eq 1; then run uenv_import; fi; sleep 3'"

}

reload_custom_envs(){

# WIP and disabled, writing custom envs again from backup

  $FW_SETENV ethaddr $ENV_ETHADDR
  if [ "$ENV_RESCUE_INSTALLED" = "1" ]; then $FW_SETENV rescue_installed $ENV_RESCUE_INSTALLED; fi
  if [ "$ENV_RESCUE_CUSTOM" != "" ]; then $FW_SETENV rescue_custom_params $ENV_RESCUE_CUSTOM; fi
  if [ "$ENV_USB_CUSTOM" != "" ]; then $FW_SETENV usb_custom_params $ENV_USB_CUSTOM; fi
  if [ "$ENV_UBIFS_CUSTOM" != "" ]; then $FW_SETENV ubifs_custom_params $ENV_UBIFS_CUSTOM; fi
}

partition_and_format_disk() {


fdisk -l

#since for various reasons I cannot get a RELIABLE way to find what device is friggin / I resort to ask the user

drives=$(fdisk -l | grep /dev | grep bytes | sed -n 1p)

if [[ "$drives" = "" ]]; then
  echo "no disks detected, exiting now"
  exit 1
fi

echo "ok, these are the drives we detected"

fdisk -l | grep /dev

echo
echo "what drive you want to install Debian in?"
disk=$(fdisk -l | grep /dev | grep bytes | awk '{if ($3 >= 1) print $2;}' | awk -F ':' '{print $1}' | sed -n 1p)
if [[ "$disk" == *"/dev"* ]]; then
  echo "1. $disk"
fi

disk=$(fdisk -l | grep /dev | grep bytes | awk '{if ($3 >= 1) print $2;}' | awk -F ':' '{print $1}' | sed -n 2p)
if [[ "$disk" == *"/dev"* ]]; then
  echo "2. $disk"
fi

disk=$(fdisk -l | grep /dev | grep bytes | awk '{if ($3 >= 1) print $2;}' | awk -F ':' '{print $1}' | sed -n 3p)
if [[ "$disk" == *"/dev"* ]]; then
echo "3. $disk"
fi

disk=$(fdisk -l | grep /dev | grep bytes | awk '{if ($3 >= 1) print $2;}' | awk -F ':' '{print $1}' | sed -n 4p)
if [[ "$disk" == *"/dev"* ]]; then
echo "4. $disk"
fi

read -r SELECTION

install_drive=$(fdisk -l | grep /dev | grep bytes | awk '{if ($3 >= 1) print $2;}' | awk -F ':' '{print $1}' | sed -n "$SELECTION"p)

install_drive_size_unit=$(fdisk -l | grep /dev | grep bytes | awk '{if ($3 >= 1) print $4;}' | sed -n "$SELECTION"p)

echo
echo "installation drive is $install_drive"
echo


disk="$install_drive"
part1="${install_drive}1"
part2="${install_drive}2"
part3="${install_drive}3"

echo
echo "---unmounting installation drive"
echo

umount -l "$part1"
umount -l "$part2"
umount -l "$part3"

echo
echo "---partitioning installation drive"
echo

#checking install drive size
install_drive_size=$(fdisk -l $install_drive | grep /dev | grep bytes | awk '{print $3}')

echo "install_drive_size is $install_drive_size"

if [ "$install_drive_size_unit" = "MB," ]; then
  #so the drive is detected in MB, we sort it depending on size

  # turning the size value into an integer
  install_drive_size_integer=$( echo "$install_drive_size" | sed 's/\..*//' )

  echo "install_drive_size_integer is $install_drive_size_integer"

  #if storage device is smaller than 1 GB the variable should remain empty
  if (( "$install_drive_size_integer" < 1024 )); then
    echo "the new rootfs drive size is less than 1 GB."
    echo "It is a bit too small for confort and will have subpar performance,"
    echo "aborting installation now."

    exit 1
  fi

  #doing a bunch of checks to decide what partition layout to create.

  if (( "$install_drive_size_integer" < 33000 )); then

    need_swap_and_data="yes"

    echo "drive is smaller than 33GB so we assume this is a flash drive."
    echo "we will create only a system partition, no swap nor data partition."

    fdisk "$disk" <<EOF
o
n
p
1


w
EOF

  else

  echo "drive is bigger than 33 GB so we assume this is a hard drive."
  echo "making a 7 GB system partition, a 1 GB swap partition and"
  echo "the rest of the space becomes a data partition"

  fdisk "$disk" <<EOF
o
n
p
1

+7168M
n
p
2

+1024M
n
p
3


w
EOF

fi

fi

if [ "$install_drive_size_unit" = "GB," ]; then
#if drive is detected in GB unit we assume it is a hard drive

  need_swap_and_data="yes"

  echo "we assume this is a hard drive."
  echo "making a 7 GB system partition, a 1 GB swap partition and"
  echo "the rest of the space becomes a data partition"

  fdisk "$disk" <<EOF
o
n
p
1

+7168M
n
p
2

+1024M
n
p
3


w
EOF

fi

fdisk -l

echo "---pausing for a bit so ioctl can re-sync partitions"
sleep 20

echo
echo "---unmounting installation drive again"
echo

umount -l "$part1"
umount -l "$part2"
umount -l "$part3"


#  make filesystems
echo "---formatting the partitions we created"

if [ -f "/sbin/e2label" ]; then
# this means we are inside a Debian rootfs,
# labelling it now as rootfs will screw up mounting later
# it will be labelled later when unmounted
mke2fs -t "ext4" "$part1"
else
#not in Debian rootfs, so we can give it a label only this way
mke2fs -t "ext4" -L "rootfs" "$part1"
fi

#checking if swap and data partition is needed
if [ "$need_swap_and_data" = "yes" ]; then
mkswap -L "swap" "$part2"
mke2fs -t "ext4" -L "data" "$part3"
fi

# this is unlikely to work in stock firmwares, but we have that binary in toolbox
echo '---doing file system check on partitons to be sure that they are ok'
e2fsck -fp "$part1"
if [ "$need_swap_and_data" = "yes" ]; then
e2fsck -fp "$part3"
fi
} # end of partition_and_format_disk

download_and_write_rootfs(){

###
#  make dirs & mount
echo "---mounting partitions"
mkdir -p "/tmp/rootfs"

fdisk -l
echo
echo "partition to mount is $part1"
sleep 5

mount "$part1" "/tmp/rootfs"

###
echo "---checking that the stuff was mounted and writable"
# Just because it's better to.
echo "testing if this is readable" > "/tmp/rootfs/testfile.txt"
if [ ! -f "/tmp/rootfs/testfile.txt" ]; then
    echo "Rootfs partition not created! Aborting!"
    exit 1
fi

#removing the test file
rm "/tmp/rootfs/testfile.txt"
echo "---everything writable"
echo

###
#  prepare the system

#downloading rootfs in the rootfs partition, aborting if fails the md5 check.

download_and_verify "$ROOTFS" "$ROOTFS_URL" "$ROOTFS_MD5"
if [ "$?" -ne "0" ]; then
        echo "## Could not install $file_dest from $file_url, exiting."
    #    if [ "$is_readonly" = "1" ]; then
    #      mount -o remount,ro /
    #    fi
        exit 1
      fi

#  extract the rootfs
echo "---extracting debian rootfs to install Debian"
cd "/tmp/rootfs/"
bunzip2 "$ROOTFS"
tar xvf "$ROOTFS_TAR"
rm "$ROOTFS_TAR"
cd
#  always make sure you're sync'd!
sync

#checking to see if it went right
if [ ! -f "/tmp/rootfs/etc/fstab" ]; then
    echo "root files not extracted correctly! Aborting!"
    exit 1
fi
echo "---everything fine"

if [ -f "/sbin/e2label" ]; then # this means we are inside a Debian rootfs
umount "$part1"
/sbin/e2label "$part1" "rootfs"
fi
}


###############################################
#####
##### DISABLED FUNCTIONS
#####
###############################################


disabling_uboot_check () {


# Dump existing uBoot and compare it to a database of known versions
###########
# we don't have stock uboots of most devices, so disabling this for now
##########

echo ""
echo "# Validating existing uBoot..."

# dump the first 512k of mtd0 to /tmp
$NANDDUMP --noecc --omitoob -l 0x80000 -f /tmp/uboot-mtd0-dump /dev/mtd0

wget -O "/tmp/valid-uboot.md5" "$VALID_UBOOT_MD5"

UPDATE_UBOOT=1
UBOOT_PLATFORM=
CURRENT_UBOOT_MD5=$(md5sum "/tmp/uboot-mtd0-dump" | cut -d' ' -f1)
UBOOT_DETAILS=$(grep $CURRENT_UBOOT_MD5 /tmp/valid-uboot.md5 | head -1)
if [ "$UBOOT_DETAILS" != "" ]; then
  UBOOT_PLATFORM=$(echo $UBOOT_DETAILS | sed 's/^\w* \(\w*\) .*$/\1/')
  UBOOT_VERSION=$(echo $UBOOT_DETAILS | sed 's/^\w* \w* \(.*\)$/\1/')
  echo "## Valid uBoot detected: [$UBOOT_PLATFORM $UBOOT_VERSION]"
else
  echo "## Unknown uBoot detected on mtd0: $CURRENT_UBOOT_MD5"
  echo "##"
  if [ "$NO_UBOOT_CHECK" != "1" ]; then
    echo "## The installer could not detect the version of your current uBoot"
    echo "## This may happen if you have installed a different uBoot on"
    echo "## /dev/mtd0 or if you have bad blocks on /dev/mtd0"
    echo "##"
    echo "## If you have bad blocks on mtd0, you should not try to install uBoot."
#    echo "##"
#    echo "## If you have installed a diffirent uBoot on mtd0, and understand the"
#    echo "## risks, you can re-run the installer with the --no-uboot-check parameter"
    echo "##"
    echo "## Installation cancelled."
    rm "/tmp/valid-uboot.md5"
    exit 1
  else
    echo "## --no-uboot-check flag detected, continuing installation"

    while [ "$UBOOT_PLATFORM" = "" ]; do
      echo ""
      echo "############################################"
      echo "Your device could not be auto-detected."
      echo ""
      echo "You must be using a device listed below to run this installer."
      echo ""
      echo "What device are you using? Type the number of your device and press ENTER."
      echo "1 - Seagate Dockstar"
      echo "2 - Seagate GoFlex Net"
      echo "3 - Seagate GoFlex Home"
      echo "4 - Pogoplug v1"
      echo "5 - Pogoplug v2 - Pink"
      echo "6 - Other"
      read -r device

      if [ "$device" = "1" ]; then
        echo "Selected Dockstar"
        UBOOT_PLATFORM="dockstar"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "2" ]; then
        echo "Selected Seagate GoFlex Net"
        UBOOT_PLATFORM="goflexnet"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "3" ]; then
        echo "Selected Seagate GoFlex Home"
        UBOOT_PLATFORM="goflexhome"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "4" ]; then
        echo "Selected Pogoplug v1"
        UBOOT_PLATFORM="pogov1"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "5" ]; then
        echo "Selected Pogoplug v2 - Pink"
        UBOOT_PLATFORM="pinkpogo"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "6" ]; then
        echo "Selected Other Device, exiting"
        echo "This installer is only compatible with the listed devices."
        exit 1
      else
        echo "Invalid Input"
      fi
    done

  fi
fi

if [ "$FORCE_PLATFORM" != "" ] && [ "$FORCE_PLATFORM" != "$UBOOT_PLATFORM" ]; then
  echo "## --force-platform paramater [$FORCE_PLATFORM] does not match detected platform [$UBOOT_PLATFORM]."
  echo "Are you sure your device is a $FORCE_PLATFORM? [y/N] "
  read -r FORCE
  if [ "$FORCE" = "y" ] || [ "$FORCE" = "Y" ]; then
    echo "## Forcing installation of [$FORCE_PLATFORM] platform"
    UBOOT_PLATFORM=$FORCE_PLATFORM
    UBOOT_VERSION="unknown"
  else
    echo "## Exiting. No changes were made to mtd0."
    exit 1
  fi
fi

UBOOT_IS_CURRENT=$(echo $UBOOT_VERSION | grep -c current)
if [ "$UBOOT_IS_CURRENT" = "1" ]; then
  echo "## The newest uBoot is already installed on mtd0."
  UPDATE_UBOOT=0
else
  UBOOT_CURRENT=$(grep $UBOOT_PLATFORM /tmp/valid-uboot.md5 | grep current | head -1 | sed 's/^\w* \w* \(.*\)-current$/\1/')
fi

rm "/tmp/valid-uboot.md5"

}

disabling_service_disabler() {

# If this is the first time this installer has been run in the
# original Pogoplug enviroment, check if the user wants to disable
# the Pogoplug services
#########
#this stuff will likely be split into device-specific functions,
#for now it stays disabled
#########

if [ -d /usr/local/cloudengines/ ] && [ ! -e $UBOOT_ORIGINAL ]; then
  killall hbwd
  echo ""
  echo ""
  echo ""
  echo "DISABLE POGOPLUG SERVICES"
  echo ""
  echo "The pogoplug service includes an auto-update feature which could"
  echo "be used to cripple or disable your device.  It is recommended"
  echo "that you disable this service."
  echo ""
  echo "NOTE: The pogoplug service is proprietary software"
  echo "created by Cloud Engines.  It is not available for use"
  echo "in other distributions and will not be available in"
  echo "your new linux installation even if you choose not to disable it."
  echo ""
  echo "Would you like to disable the pogoplug services? [Y/n] "
  read -r DISABLE
  if [ "$DISABLE" = "" ] || [ "$DISABLE" = "y" ] || [ "$DISABLE" = "Y" ];
  then

    echo "Applying fixes to the pogoplug environment..."

    mount -o rw,remount /

    # Add /sbin to the path and cleanup the shell prompt
    if [ ! -f "/root/.bash_profile" ]; then
      echo -e \
  "export PS1='\h:\w\$ '
  export PATH='/usr/bin:/bin:/sbin'
  " > /root/.bash_profile
    fi

    chmod go+w /dev/null

    # Re-enable dropbear (updated dockstars only)
    sed -i 's/^#\/usr\/sbin\/dropbear/\/usr\/sbin\/dropbear/' /etc/init.d/db > /dev/null 2>&1

    echo "Disabling the pogoplug service..."
    # Comment out the line that starts hmbgr.sh
    sed -i 's/^\/etc\/init.d\/hbmgr.sh start/#Uncomment the line below to enable the pogoplug service\n#\/etc\/init.d\/hbmgr.sh start/' /etc/init.d/rcS

    mount -o ro,remount /

    echo "Done fixing pogoplug environment."
    echo ""
  fi

  UBOOT_ORIGINAL_URL="$UBOOT_MTD0_BASE_URL.$UBOOT_PLATFORM.original.kwb"
  install "$UBOOT_ORIGINAL"   "$UBOOT_ORIGINAL_URL"    644

  #install "$BLPARAM"          "$BLPARAM_URL"           755 we don't have blparam

  if   [ "$UBOOT_PLATFORM" = "pinkpogo"  ]; then BOOTCMD='nand read.e 0x800000 0x100000 0x200000; setenv bootargs $(console) $(bootargs_root); bootm 0x800000'
  # dockstar, goflex, and pogov1 have the same bootcmd
  else BOOTCMD='nand read.e 0x800000 0x100000 0x300000; setenv bootargs $(console) $(bootargs_root); bootm 0x800000'
  fi
  #$BLPARAM "bootcmd=$BOOTCMD" > /dev/null 2>&1

  # Preserve the MAC address
  #ENV_ETHADDR=`$BLPARAM | grep "^ethaddr=" | cut -d'=' -f 2-`
fi
}

################################################
#####
##### MAIN
#####
################################################

#doing a root check, because the installer must be run as root for obvious reasons
if [ "$(id -u)" -eq 0 ]; then
	echo ;
else
	echo "you must run this script as root";
exit
fi

echo ""
echo ""
echo "!!!!!!  WARNING: PLEASE READ THIS CAREFULLY  !!!!!!"
echo ""
echo "To boot Debian we will replace the bootloader in the "
echo "internal memory of the device"
echo ""
echo "Due to bootloader limitations, booting the stock firmware"
echo "will usually not be possible afterwards."
echo "Booting third party firmwares like OpenWRT or LEDE will work"
echo "but installing them is outside the scope of this installer."
echo "At least for now."
echo ""
echo "If you lose power while the bootloader is being flashed,"
echo "your device could be left in an unusable state and require"
echo "serial connection to recover, please ask for help in the forums."
echo ""
echo ""
echo "We also need a storage device to install Debian in as the onboard"
echo "storage of most devices is tiny, please make sure that there is a"
echo "USB or Sata storage device we can install Debian in."
echo "We need at least 1GB of storage space, the faster storage the better."
echo ""
echo "This script will DESTROY ALL EXISTING DATA ON THE STORAGE DEVICE"
echo "so move all your data off it please."
echo ""
echo ""
echo "By typing ok, you agree to assume all liabilities and risks"
echo "associated with running this installer."
echo ""
echo "If everything looks good, type 'ok' to continue: "


read -r IS_OK
if [ "$IS_OK" != "OK" ] && [ "$IS_OK" != "Ok" ] && [ "$IS_OK" != "ok" ]
then
  echo "Exiting..."
  exit
fi

#SELECTING DEVICE
echo "Good, now we need to know what device we are working on"
echo "here is a list, please write the number of the device"

#add here all devices supported
echo "00. download and unpack toolbox"
echo "01. make a Debian rootfs in a drive"
echo "02. test chroot with a printenv"
echo "1.Zyxel nsa310"
echo "2.Zyxel nsa325v1 or nsa325v2"
echo "3.Pogoplug"

#loading the user selection
read -r DEVICE_TYPE

# and here goes all device-specific operations
case "$DEVICE_TYPE" in


    00)    echo "downloading tools"
          download_tools
          echo "tools downloaded in $TOOLS, busybox tools installed in $BUSYBOX_INSTALL"
          echo "you may want to write " "export PATH="$TOOLS":"$BUSYBOX_INSTALL":"$PATH""
          echo "so you can use them as default tools for this session"
    ;;

    01)    echo "downloading tools and making a rootfs"
          download_tools
          partition_and_format_disk
          download_and_write_rootfs
    ;;
    02)    echo "downloading tools and testing chroot"
          download_tools
          echo "######this is the output of normal fw_printenv"
          echo "######this is the output of normal fw_printenv"
          echo "######this is the output of normal fw_printenv"
          echo "######this is the output of normal fw_printenv"
          #getting the path back to default
          export PATH="$ORIGINAL_PATH"
          fw_printenv

          echo "######this is the output of chrooted fw_printenv"
          echo "######this is the output of chrooted fw_printenv"
          echo "######this is the output of chrooted fw_printenv"
          echo "######this is the output of chrooted fw_printenv"
          #using our tools again
          export PATH="$TOOLS"
          run_fw_printenv_setenv_in_chroot fw_printenv
    ;;


	1) #zyxel nsa310
	echo "disabled"
	exit 1
		UBOOT_PLATFORM="nsa310"
		STOCK_MTD0="/dev/mtd0" #this is the how the mtd0 is, as seen from inside the stock firmware
		ENV_DTB_FILE="'/boot/dts/kirkwood-dockstar.dtb'"
		download_tools
        partition_and_format_disk
		download_and_write_rootfs

		preserve_crucial_envs_for_later

		install_envs
		#install uboot must be kept last, so that error in partitioning or writing the rootfs don't require recovery
		install_uboot

		set_crucial_envs

	;;
	2) #zyxel nsa325
    echo "disabled"
	exit 1
		UBOOT_PLATFORM="nsa325"
		STOCK_MTD0="/dev/mtd0" #this is the how the mtd0 is, as seen from inside the stock firmware
		ENV_DTB_FILE="'/boot/dts/kirkwood-dockstar.dtb'"
		download_tools
        partition_and_format_disk
		download_and_write_rootfs
		install_uboot
	;;
	3) #Pogoplug
    echo "disabled"
	exit 1
		UBOOT_PLATFORM="Pogoplug"
		STOCK_MTD0="/dev/mtd0" #this is the how the mtd0 is, as seen from inside the stock firmware
		ENV_DTB_FILE="'/boot/dts/kirkwood-dockstar.dtb'"

		#device-specific commands to be run before proceding
		# Stop the pogoplug engine
		killall -q hbwd

		download_tools
        partition_and_format_disk
		download_and_write_rootfs
		install_uboot

	;;

esac


#getting the path back to default
export PATH="$ORIGINAL_PATH"




echo ""
echo ""
echo ""
echo ""
echo "Installation complete"
echo ""
echo "You can now reboot your device into Debian."
echo "If your device does not start Debian after rebooting,"
echo "you may need to restart the device by disconnecting the power."
echo ""
echo "The new root password is 'root'  Please change it immediately after"
echo "logging in."
echo ""
echo "Reboot now? [Y/n] "

read -r IN
if [ "$IN" = "" ] || [ "$IN" = "y" ] || [ "$IN" = "Y" ]
then
  /sbin/reboot
fi

exit 0


